require 'ScintillaBridge.rb'
require 'TraceDebug.rb'



class EditControl < ScintillaBridge

  def initialize( editWindowHandle )
    super( editWindowHandle )
    
    setUpEditor()
  end

  # predefined color (from Color, but don't seem to works)
  Black  = Color.new( 0x000000 )    
  Silver = Color.new( 0xC0C0C0 )  
  Gray   = Color.new( 0x808080 )   
  White  = Color.new( 0xFFFFFF )   
  Maroon = Color.new( 0x800000 )   
  Red    = Color.new( 0xFF0000 )   
  Purple = Color.new( 0x800080 )   
  Fuchsia= Color.new( 0xFF00FF )    
  Green  = Color.new( 0x008000 )
  Lime   = Color.new( 0x00FF00 )
  Olive  = Color.new( 0x808000 )
  Yellow = Color.new( 0xFFFF00 )
  Navy   = Color.new( 0x000080 )
  Blue   = Color.new( 0x0000FF )
  Teal   = Color.new( 0x008080 )
  Aqua   = Color.new( 0x00FFFF )
  AquaBlue = Color.new( 0x0080C0 )
  
  def setUpEditor

    # use RUBY lexer
	self.lexer = SCLEX_RUBY					# weird, it seems that 'lexer = SCLEX_RUBY' declare
                                            # declare a local variable instead of calling lexer=()
											# => to investigate, using self. in the meanwhile

    # set RUBY keywords
    setKeyWords( 0, %q[
__LINE__ __FILE__ BEGIN END alias and begin break case class def defined? do 
else elsif end ensure false for if in module next nil not or 
redo rescue retry return self super then true undef unless until 
when while yield' ] );

    # set a flashy color for all styles, so we can spot those we forgot to set...
	for style in 0..31
      setStyle( style, Yellow );
	end

	# setting up style colors
    setStyle( SCE_P_STRING, Red );           # "..."
    setStyle( SCE_P_CHARACTER, Purple );     # '...'

    setStyle( STYLE_DEFAULT, Black, White, 10, "Courrier New" );
    setStyle( SCE_P_DEFAULT, Black, White, 10, "Courrier New" );

    setStyle( SCE_P_WORD, Blue );            # for keywords: class def module...

    setStyle( SCE_P_CLASSNAME, Teal );       # name following def, class or module
	setStyle( SCE_P_DEFNAME, Navy );         # name following def, class or module

    setStyle( SCE_P_STRING, Red );           # "..."
    setStyle( SCE_P_CHARACTER, Purple );     # '...'

    setStyle( SCE_P_OPERATOR, AquaBlue );    # + - / = ...

    setStyle( SCE_P_NUMBER, Maroon );        # 123.456

    setStyle( SCE_P_TRIPLEDOUBLE, Green );   # =begin ... =end
    setStyle( SCE_P_COMMENTLINE, Green );    # # ...

    setStyle( SCE_P_IDENTIFIER, Black );     # identifiers

    # add some text
	text = %q[#one line comment
=begin
multiline comment
=end
class MyClass
  def initialize( lastName )
    @name = 'Turtle' + \" #{lastName}\"
    @age = 3* 21 /7;
    if @name.length < age then
      @name.collect do |c| @age += 1 end
    end
    if @name =~ /Vieux/ then @age += 12 end
    @friends = %w( bert albert jolibert )
    if friends =~ %r( bert ) then @age -=7 end
  end
end

# this is a module
module Find
end ]

    addText( text )

	setMarginTypeN( 0, SC_MARGIN_NUMBER );
	setMarginWidthN( 0, 32 );

#define SCMOD_SHIFT 1
#define SCMOD_CTRL 2
#define SCMOD_ALT 4

    self.undoCollection = true

  end

  def setStyle( style, fore, back = White, size = nil, face = nil )
    styleSetFore( style, fore )
    styleSetBack( style, back )
	styleSetSize( style, size ) if size
	styleSetFont( style, face ) if face
  end

  def eventCharAdded( ch )
    if ch == "\r"  ||  ch == "\n"
	  smartIndent()
	else
	end
  end

  def smartIndent()
	if currentLineNumber() > 0  &&  lineLength( currentLineNumber() ) <= 2      # 2 <= '\r\n'
	  previousLine = getLine( currentLineNumber() -1 )
	  indent = /^\s*/.match( previousLine )[0]
	  replaceSel( indent )
	  TraceDebug.puts( "Replacing selection with : #{indent.inspect()}" );
	end
  end

end
