
require 'TraceDebug'
require 'EditControl'


class ActionStatusUpdater
# def enable( enabled )
end

# should probably be a module,mixin or whatever (need read doc)
class Action
  def execute()
  end

  def updateStatus( updater )
    updater.enable( false )
  end
end


class EditorAction < Action
  def initialize( mainFrame )
    @mainFrame = mainFrame
  end

  def updateStatus( updater )
    updater.enable= enabled?( @mainFrame.activeWindow )
  end

  # only called if enabled? returns true
  def execute()
    editor = @mainFrame.activeWindow
    executeEditorAction( editor )
  end

  # called with a valid editor
  def executeEditorAction( editor )
  end

  def enabled?( editor )
    editor != nil  #&&  editor.kind_of?( EditControl )
  end
end


class SimpleEditorAction < EditorAction
  def initialize( mainFrame, executeMethod, enabledMethod =nil )
    super( mainFrame )
	@executeMethod = executeMethod
	@enabledMethod = enabledMethod
  end

  def enabled?( editor )
    super( editor )  &&  
      ( @enabledMethod ? editor.send( @enabledMethod ) : true )
  end

  def executeEditorAction( editor )
    editor.send( @executeMethod )
  end
end


class MainFrame

  attr_reader :activeWindow

  def initialize()
    @actions = Hash.new
    @activeWindow = nil

	registerDefaultActions()
  end

  def activeWindow=( window )
    @activeWindow = window
	TraceDebug.puts "Activated window <%s>" % window.inspect()
  end

  def actionFor( actionId )
#    TraceDebug.puts "Getting action for <%d>" % actionId
    @actions[actionId]
  end

  def registerAction( actionId, action )
    TraceDebug.puts "Registering action for <%d>" % actionId
    @actions[actionId] = action
  end

  def unregisterAction( actionId )
    @actions.delete(actionId)
  end

	# hardcoded for now...

	ID_FILE_NEW      =0xE100
	ID_FILE_OPEN     =0xE101
	ID_FILE_CLOSE    =0xE102
	ID_FILE_SAVE     =0xE103
	ID_FILE_SAVE_AS  =0xE104
	ID_EDIT_COPY     =0xE122
	ID_EDIT_CUT      =0xE123
	ID_EDIT_PASTE    =0xE125
	ID_EDIT_UNDO     =0xE12B
	ID_EDIT_REDO     =0xE12C

  def registerDefaultActions()
    registerAction( ID_EDIT_COPY, SimpleEditorAction.new( self, :copy ) )
    registerAction( ID_EDIT_CUT, SimpleEditorAction.new( self, :cut ) )
    registerAction( ID_EDIT_PASTE, SimpleEditorAction.new( self, :paste, :canPaste ) )
    registerAction( ID_EDIT_UNDO, SimpleEditorAction.new( self, :undo, :canUndo ) )
    registerAction( ID_EDIT_REDO, SimpleEditorAction.new( self, :redo, :canRedo ) )
  end

end